/*
 * Copyright (c) Huawei Technologies Co., Ltd. 2020-2024. All rights reserved.
 */

#ifndef DPC_IMPL_TYPE_CONTROLWAYPOINTS_H
#define DPC_IMPL_TYPE_CONTROLWAYPOINTS_H

#include <cfloat>
#include <cmath>
#include "dpc/impl_type_msgheader.h"
#include "dpc/impl_type_vector_controlwaypointstruct.h"
#include "dpc/impl_type_vector_remindwaypointstruct.h"
#include "dpc/impl_type_vector_transitlanestruct.h"
#include "impl_type_float.h"

namespace dpc {
struct ControlWaypoints {
    ::dpc::MsgHeader msg_header;
    ::dpc::vector_ControlWayPointStruct control_waypoints_list;
    ::dpc::vector_RemindWayPointStruct reminder_waypoint;
    ::dpc::vector_TransitLaneStruct transit_lanes_list;
    ::Float remainDistanceMajorPath{ 0.0F };

    static bool IsPlane()
    {
        return false;
    }


    using IsEnumerableTag = void;
    template<typename F>
    void enumerate(F& fun)
    {
        fun(msg_header);
        fun(control_waypoints_list);
        fun(reminder_waypoint);
        fun(transit_lanes_list);
        fun(remainDistanceMajorPath);
    }

    template<typename F>
    void enumerate(F& fun) const
    {
        fun(msg_header);
        fun(control_waypoints_list);
        fun(reminder_waypoint);
        fun(transit_lanes_list);
        fun(remainDistanceMajorPath);
    }

    template<typename F>
    void enumerate_internal(F& fun)
    {
        fun("msg_header", msg_header);
        fun("control_waypoints_list", control_waypoints_list);
        fun("reminder_waypoint", reminder_waypoint);
        fun("transit_lanes_list", transit_lanes_list);
        fun("remainDistanceMajorPath", remainDistanceMajorPath);
    }

    template<typename F>
    void enumerate_internal(F& fun) const
    {
        fun("msg_header", msg_header);
        fun("control_waypoints_list", control_waypoints_list);
        fun("reminder_waypoint", reminder_waypoint);
        fun("transit_lanes_list", transit_lanes_list);
        fun("remainDistanceMajorPath", remainDistanceMajorPath);
    }

    friend bool operator==(const ::dpc::ControlWaypoints& lhs, const ::dpc::ControlWaypoints& rhs) noexcept
    {
        return (lhs.msg_header == rhs.msg_header) && (lhs.control_waypoints_list == rhs.control_waypoints_list) && (lhs.reminder_waypoint == rhs.reminder_waypoint) && (lhs.transit_lanes_list == rhs.transit_lanes_list) && (fabs(static_cast<double>(lhs.remainDistanceMajorPath - rhs.remainDistanceMajorPath)) < DBL_EPSILON);
    }
};
} // namespace dpc


#endif // DPC_IMPL_TYPE_CONTROLWAYPOINTS_H
